/*
 * CloseButton.java:  a Runnable Button to close a Window
 * for the Nomencurator, a Nomenclature Heuristic Model.
 *
 * Copyright (c) 2000 Nozomi `James' Ytow
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: CloseButton.java,v 1.1.1.1 2002/01/16 12:33:33 ryo Exp $
 *	$Log: CloseButton.java,v $
 *	Revision 1.1.1.1  2002/01/16 12:33:33  ryo
 *	initial import into CVS
 *	
 *	
 *
 */

package org.nomencurator.awt;

import java.awt.Window;

import jp.kyasu.graphics.Text;
import jp.kyasu.graphics.VAbstractButton;
import jp.kyasu.graphics.VButton;
import jp.kyasu.graphics.Visualizable;
import jp.kyasu.graphics.VText;

import org.nomencurator.awt.Button;

public  class CloseButton
    extends Button
{

    protected Window window;

    public CloseButton()
    {
	this((Window)null);
    }

    public CloseButton(Window window)
    {
	this("Close", window);
    }

    /**
     * Constructs a button with the specified label.
     * @param label A string label for the button.
     */
    public CloseButton(String label)
    {
	this(label, null);
    }

    public CloseButton(String label, Window window)
    {
	super(label);
	setWindow(window);
    }

    /**
     * Constructs a button with the specified text.
     * @param text A text label for the button.
     */
    public CloseButton(Text text)
    {
	this(text, null);
    }

    public CloseButton(Text text, Window window)
    {
	super(text);
	setWindow(window);
    }

    /**
     * Constructs a button with the specified visual object.
     * @param visualizable A visual label for the button.
     */
    public CloseButton(Visualizable visualizable)
    {
	this(visualizable, null);
    }

    public CloseButton(Visualizable visualizable, Window window)
    {
	super(visualizable);
	setWindow(window);
    }

    /**
     * Constructs a button with the specified visual button.
     * @param button A visual button for the button.
     */
    public CloseButton(VAbstractButton button)
    {
	this(button, null);
    }

    public CloseButton(VAbstractButton button, Window window)
    {
	super(button);
	setWindow(window);
    }

    public void setWindow(Window window)
    {
	this.window = window;
    }

    public Window getWindow()
    {
	return window;
    }

    public void run()
    {
	if(window != null)
	    window.dispose();
    }

}
